<?php
/**
 * Plugin Name: Emag Marketplace Connector
 * Plugin URI: https://zitec.com/
 * Description: WooCommerce Plugin for Emag Marketplace
 * Version: 1.1.0
 * Author: Zitec
 * Author URI: https://zitec.com
 * Copyright: © 2017 Zitec.
 * Requires at least: 4.7
 * Tested up to: 4.9
 * Depends: Woocommerce
 * Text Domain: emkp
 **
 * @package WooCommerce Emag Marketplace
 * @category Core
 * @author Zitec
 */

require_once ABSPATH . 'wp-admin/includes/plugin.php';

if ( version_compare( PHP_VERSION, '5.6.0' ) < 0 ) {
    deactivate_plugins( plugin_basename( __FILE__ ) );
    wp_die( __( 'This plugin requires minimum PHP 5.6.0' ) . '<br/><a href="' . admin_url( 'plugins.php' ) . '">&laquo; ' . __( 'Return to Plugins' ) . '</a>' );
}

if ( ! class_exists( 'Emkp' ) ) :

    class Emkp {

        /**
         * Emkp version.
         *
         * @var string
         */
        public $version = '1.1.0';

        /**
         * The single instance of the class.
         *
         * @var Emkp
         */
        protected static $_instance = null;

        /**
         * @var \Zitec\MkpApiWrapper\InstantiableClient[]
         */
        private $client_instances = [];

        /**
         * @var string
         */
        private $migrations_directory;

        /**
         * @var int
         */
        private $migration_version = 2;

        /**
         * Emkp Constructor.
         */
        public function __construct() {
            $this->migrations_directory = __DIR__ . DIRECTORY_SEPARATOR . 'includes' . DIRECTORY_SEPARATOR . 'migrations' . DIRECTORY_SEPARATOR;
            
            $this->define_constants();

            $this->includes();
            $this->init_hooks();
            $this->init_apiwrapper();

            if ( is_admin() ) {
                $this->run_migrations();
            }

            $this->init_crons();

            add_action( 'plugins_loaded', [ $this, 'load_text_domain' ] );
        }

        public function load_text_domain() {
            load_plugin_textdomain( 'emkp', false, basename( dirname( __FILE__ ) ) . '/i18n/languages/' );
        }


        /**
         * Define plugin constants
         */
        private function define_constants() {
            $this->define( 'EMKP_MIN_WORDPRESS_VERSION', '4.7' );
            $this->define( 'EMKP_MIN_WOOCOMMERCE_VERSION', '3.0' );
            $this->define( 'EMKP_MIN_WOOCOMMERCE_MAIN_FILE', 'woocommerce/woocommerce.php' );
            $this->define( 'EMKP_ABSPATH', dirname( __FILE__ ) . '/' );
            $this->define( 'EMKP_POST_ACTION_CATEGORIES_IMPORT_UPDATE', 'emkp_post_action_categories_import_update' );
        }

        /**
         * Define constant if not already set.
         *
         * @param  string $name
         * @param  string|bool $value
         */
        private function define( $name, $value ) {
            if ( ! defined( $name ) ) {
                define( $name, $value );
            }
        }

        /**
         * Include needed files
         */
        public function includes() {
            // Api wrapper files & extensions
            require_once( EMKP_ABSPATH . 'vendor/autoload.php' );
            require_once( EMKP_ABSPATH . 'includes/core/class-wp-list-table.php' );

            require_once( EMKP_ABSPATH . 'includes/apiwrapper/class-emkp-email-alert-manager.php' );
            require_once( EMKP_ABSPATH . 'includes/apiwrapper/class-emkp-file-logger.php' );

            require_once( EMKP_ABSPATH . 'includes/class-emkp-general-settings.php' );
            require_once( EMKP_ABSPATH . 'includes/mapping/class-emkp-categories.php' );
            require_once( EMKP_ABSPATH . 'includes/mapping/class-emkp-categories-characteristics-edit.php' );

            require_once( EMKP_ABSPATH . 'includes/class-emkp-product-sender.php' );
            require_once( EMKP_ABSPATH . 'includes/class-emkp-product-tab.php' );
            require_once( EMKP_ABSPATH . 'includes/mapping/class-emkp-mapping-list.php' );
            require_once( EMKP_ABSPATH . 'includes/mapping/class-emkp-mapping-characteristics-list.php' );
            require_once( EMKP_ABSPATH . 'includes/class-emkp-order-update.php' );
            require_once( EMKP_ABSPATH . 'includes/class-emkp-invoice-handler.php' );
            require_once( EMKP_ABSPATH . 'includes/class-emkp-awb-handler.php' );
            require_once( EMKP_ABSPATH . 'includes/class-emkp-awb-list.php' );
            require_once( EMKP_ABSPATH . 'includes/class-emkp-localities-handler.php' );
        }

        public function init_static_content() {
            $js_files = [
                'assets/js/select2.min.js',
                'assets/js/invoices_handler.js',
                'assets/js/shipping_handler.js',
                'assets/js/localities_handler.js',
                'assets/js/scripts.js'
            ];
            foreach ( $js_files as $file ) {
                wp_register_script( md5( $file ), plugins_url( $file, __FILE__ ), [ 'jquery' ], $this->version );
                wp_enqueue_script( md5( $file ) );
            }

            $css_files = [ 'assets/css/select2.min.css', 'assets/css/emkpstyle.css', ];
            foreach ( $css_files as $file ) {
                wp_enqueue_style( md5( $file ), plugins_url( $file, __FILE__ ), [], $this->version );
            }
        }

        /**
         * Add general settings in menu and init page
         */
        public static function emkp_admin_actions() {
            // Menu
            add_menu_page( 'Emag Marketplace', 'Emag Marketplace', '', 'emkp_general', '', 'dashicons-store' );

            $class_emkp_general_settings = new Emkp_General_Settings();
            $class_emkp_general_settings->init();

            $class_emkp_categories = new EMKP_Categories();
            $class_emkp_categories->init();
            $categories_characteristics_edit = new EMKP_Categories_Characteristics_Edit();
            $categories_characteristics_edit->init();
        }

        /**
         * Main Emag Markeplace Instance.
         *
         * Ensures only one instance of Emag Markeplace is loaded or can be loaded.
         */
        public static function instance() {

            if ( is_null( self::$_instance ) ) {
                self::$_instance = new self();
            }

            return self::$_instance;
        }

        /**
         * Hook into actions and filters.
         */
        private function init_hooks() {
            register_activation_hook( __FILE__, array( $this, 'emkp_activate' ) );
            register_deactivation_hook(__FILE__, array( $this, 'emkp_deactivate' ) );
            register_uninstall_hook( __FILE__, [ get_class( $this ), 'emkp_delete' ] );

            // If Woocommerce active , add admin menu and pages
            if ( is_plugin_active( EMKP_MIN_WOOCOMMERCE_MAIN_FILE ) ) {
                add_action( 'admin_menu', array( $this, 'emkp_admin_actions' ) );
                add_action( 'admin_enqueue_scripts', array( $this, 'init_static_content' ), 99 );
            } // If Woocommerce inactive , deactivate Emag Marketplace Plugin
            else {
                deactivate_plugins( plugin_basename( __FILE__ ) );
            }

            // Add new emag order
            add_action( 'wp_loaded', array( $this, 'emkp_save_new_order' ) );
        }

        public function init_crons() {
            add_filter( 'cron_schedules', array( $this, 'cron_add_interval' ) );

            add_action( 'wp_loaded', array( $this, 'emkp_process_products_activation' ) );
            add_action( 'emkp_process_products', array( $this, 'emkp_process_products_init' ) );

            add_action( 'wp_loaded', array( $this, 'emkp_import_orders_activation' ) );
            add_action( 'emkp_import_orders', array( $this, 'emkp_import_orders_init' ) );

            add_action( 'wp_loaded', array( $this, 'emkp_delete_invoices_activation' ) );
            add_action( 'emkp_delete_invoices', array( $this, 'emkp_delete_invoices_init' ) );
        }

        /**
         * Schedule products synchronisation cron
         */
        public function emkp_process_products_activation() {
            if ( ! wp_next_scheduled( 'emkp_process_products' ) ) {
                wp_schedule_event( time(), 'every10minute', 'emkp_process_products' );
            }
        }

        /**
         * Schedule order importing cron
         */
        public function emkp_import_orders_activation() {
            if ( ! wp_next_scheduled( 'emkp_import_orders' ) ) {
                wp_schedule_event( time(), 'every10minute', 'emkp_import_orders' );
            }
        }

        /**
         * Schedule delete invoices older than 30 days.
         */
        public function emkp_delete_invoices_activation() {
            if ( ! wp_next_scheduled( 'emkp_delete_invoices' ) ) {
                wp_schedule_event( strtotime( '03:30' ), 'daily', 'emkp_delete_invoices' );
            }
        }

        /**
         * Process products cron function
         */
        public function emkp_process_products_init() {
            $this->emkp_cron_init( 'process-products' );
        }

        /**
         * Import orders cron function
         */
        public function emkp_import_orders_init() {
            $this->emkp_cron_init( 'import-emkp-orders' );
        }

        /**
         * Delete old invoices cron function
         */
        public function emkp_delete_invoices_init() {
            $this->emkp_cron_init( 'delete-old-invoices' );
        }

        /**
         * @param $cron_method
         */
        public function emkp_cron_init( $cron_method ) {
            $cron_files = [
                'process-products'    => EMKP_ABSPATH . 'includes/crons/process-products.php',
                'import-emkp-orders'  => EMKP_ABSPATH . 'includes/crons/import-emkp-orders.php',
                'delete-old-invoices' => EMKP_ABSPATH . 'includes/crons/delete-old-invoices.php',
            ];

            if ( $cron_method && array_key_exists( $cron_method, $cron_files ) ) {
                $cron_data = include $cron_files[ $cron_method ];

                if ( isset( $cron_data['command_name'], $cron_data['callback'] ) && is_callable( $cron_data['callback'] ) ) {
                    $cron = $cron_data['callback'];
                    call_user_func( $cron );
                }
            }
        }

        /**
         * @param $schedules
         *
         * @return mixed
         */
        public function cron_add_interval( $schedules ) {
            // Adds once every minute to the existing schedules.
            $schedules['every10minute'] = array(
                'interval' => 600,
                'display'  => __( 'Once Every 10 Minutes' )
            );

            return $schedules;
        }

        /**
         * Url for Emkp to ping with new order
         *  After insert sends back acknowledgement notification
         */
        public function emkp_save_new_order() {
            global $wpdb;

            $url = strtok( $_SERVER["REQUEST_URI"], '?' );
            
            if ( ! array_key_exists( 'REQUEST_METHOD', $_SERVER ) || $_SERVER['REQUEST_METHOD'] !== 'GET' || strpos($url,'index.php') === false || ! isset ( $_GET["type"] ) || $_GET["type"]!=='emkp_new_order' || ! isset ( $_GET["order_id"] ) || ! is_numeric( $_GET["order_id"] ) || $_GET["order_id"] <= 0 ) {
                return;
            }

            $tablename = $wpdb->prefix . 'emkp_orders_queue';

            $order_id = (int) esc_sql( $_GET['order_id'] );
            
            $wpdb->insert( $tablename, array(
                    'emag_order_id' => $order_id
                )
            );

            exit;
        }
        
        /**
         * Disable scheduled tasks
         */
        public function emkp_deactivate() {
            $crons = array('emkp_process_products', 'emkp_import_orders', 'emkp_delete_invoices','emkp_crons' );
            foreach ($crons as $cron) {
                $timestamp = wp_next_scheduled ($cron);
                wp_unschedule_event ($timestamp, $cron);    
            }
        }
        
        /**
         * Verify PHP Wordpress version and Woocommerce plugin and version before activation
         */
        public function emkp_activate() {
            global $wp_version;
            $error_message = '';

            // Verify that Wordpress version is equal or greater than minimum required version
            if ( version_compare( $wp_version, EMKP_MIN_WORDPRESS_VERSION, '<' ) ) {

                $error_message .= __( 'Sorry, but this plugin requires the Wordpress Version greater than ' ) . EMKP_MIN_WORDPRESS_VERSION . '.<br/>';
            }

            // Verify that Woocomerce plugin is present and active
            if ( ! is_plugin_active( EMKP_MIN_WOOCOMMERCE_MAIN_FILE ) ) {
                $error_message .= __( 'Sorry, but this plugin requires the Woocommerce plugin installed and activated.' ) . '<br/>';
            } // Verify that Woocomerce version is equal or greater than minimum required version
            else {
                $woocomerce_data = get_plugin_data( WP_PLUGIN_DIR . '/' . EMKP_MIN_WOOCOMMERCE_MAIN_FILE );

                if ( ! version_compare( $woocomerce_data['Version'], EMKP_MIN_WOOCOMMERCE_VERSION, '>=' ) ) {
                    $error_message .= __( 'Sorry, but this plugin requires the Woocommerce Version greater than ' ) . EMKP_MIN_WOOCOMMERCE_VERSION . '. <br/>';
                }
            }

            if ( ! empty( $error_message ) ) {
                wp_die( $error_message . '<br/><a href="' . admin_url( 'plugins.php' ) . '">&laquo; ' . __( 'Return to Plugins' ) . '</a>' );
            }

            if ( empty( $error_message ) ) {
                require_once( EMKP_ABSPATH . 'includes/class-emkp-schema-updater.php' );
                ( new Emkp_Schema_Updater () )->install_schema();
            }
        }

        public function init_apiwrapper() {
            $this->get_client_instance( get_option( 'emkp_api_url' ), get_option( 'emkp_username' ), get_option( 'emkp_password' ), false, false );
        }

        public static function emkp_delete() {
            self::emkp_deactivate();
            
            require_once( EMKP_ABSPATH . 'includes/class-emkp-schema-updater.php' );
            ( new Emkp_Schema_Updater () )->uninstall_schema();
        }

        /**
         * @param string $api_url
         * @param string $api_username
         * @param string $api_password
         * @param bool $shared
         * @param bool $instantiable
         *
         * @return \Zitec\MkpApiWrapper\InstantiableClient
         */
        public function get_client_instance( $api_url, $api_username, $api_password, $shared = true, $instantiable = true ) {
            global $wp_version;

            $key = md5( $api_url . $api_username . $api_password . (int) $instantiable );

            if ( $shared && array_key_exists( $key, $this->client_instances ) ) {
                return $this->client_instances[ $key ];
            }

            /** @var \Zitec\MkpApiWrapper\InstantiableClient $client */
            if ( $instantiable ) {
                $client = new \Zitec\MkpApiWrapper\InstantiableClient( $api_url, $api_username, $api_password, new Emkp_FileLogger(), new Emkp_Email_Alert_Manager() );
            } else {
                $client = \Zitec\MkpApiWrapper\Client::init( $api_url, $api_username, $api_password, new Emkp_FileLogger(), new Emkp_Email_Alert_Manager() );
            }

            $client->setDebugInfo( get_site_url(), 'wordpress', $wp_version, $this->version );

            $this->client_instances[ $key ] = $client;

            return $client;
        }

        /**
         * Run db migrations
         * 
         * @throws Exception
         */
        private function run_migrations() {
            $key = 'emkp_migration_version';
            $current_version = get_option( $key, 0 );

            if ( $current_version < $this->migration_version ) {
                require_once $this->migrations_directory . 'emkp-abstract-migration.php';

                for ( $i = $current_version + 1; $i <= $this->migration_version; $i ++ ) {
                    $migration_number = str_pad( $i, 3, '0', STR_PAD_LEFT );
                    $file             = $this->migrations_directory . 'migration-' . $migration_number . '.php';

                    if ( ! file_exists( $file ) ) {
                        throw new Exception( sprintf( 'Missing migration file %s', $file ) );
                    }
                    
                    require_once $file;

                    $migration_class_name = 'Emkp_Migration_Version_' . $migration_number;

                    if ( ! class_exists( $migration_class_name, false ) ) {
                        throw new Exception( sprintf( 'Missing migration class %s', $migration_class_name ) );
                    }

                    /** @var Emkp_Abstract_Migration $migration_instance */
                    $migration_instance = new $migration_class_name;

                    if ( ! ( $migration_instance instanceof Emkp_Abstract_Migration ) ) {
                        throw new Exception( sprintf( 'Migration class %s must extend the %s class.', $migration_class_name, 'Emkp_Abstract_Migration' ) );
                    }

                    $migration_instance->run();

                    update_option( $key, $i );
                }
            }

        }
    }
endif;

function Emkp() {
    return Emkp::instance();
}

// Global for backwards compatibility.
$GLOBALS['emkp'] = Emkp();
