<?php
if ( ! class_exists( 'Emkp_General_Settings' ) ) :

    /**
     * Class Emkp_General_Settings
     */
    class Emkp_General_Settings {
        /**
         * List with allowed locales
         *
         * @var array
         */
        protected $locales;

        /**
         * List with allowed currencies
         *
         * @var array
         */
        protected $currencies;

        /**
         * List with allowed order statuses
         *
         * @var array
         */
        protected $emkp_order_statuses;

        /**
         * List with allowed shipping methods
         *
         * @var array
         */
        protected $emkp_shipping_methods;

        /**
         * Initialize settings page and variables
         */
        public function init() {
            // Add page to menu
            add_submenu_page( 'emkp_general', __('Emag Marketplace'), __('Settings'), 'manage_options', 'empk_general_settings', [
                $this,
                'page'
            ] );

            $this->emkp_order_statuses   = wc_get_order_statuses();
            $this->emkp_shipping_methods = WC_Shipping::instance()->get_shipping_methods();

            $this->locales    = [
                'ro_RO' => __( 'Romanian' ),
                'bg_BG' => __( 'Bulgarian' ),
                'hu_HU' => __( 'Hungarian' ),
                'pl_PL' => __( 'Polish' )
            ];

            $this->currencies = [ 'RON', 'BGN', 'HUF', 'PLN' ];
        }

        /**
         * Validate & update options submitted
         *
         * @return array
         */
        public function save_emkp_settings_data() {
            $invalid_fields = [];

            // If not POST, nothing to verify
            if ( $_SERVER['REQUEST_METHOD'] !== 'POST' ) {
                return $invalid_fields;
            }

            if ( isset( $_POST['submit'] ) ) {
                // No need to process submit
                unset( $_POST['submit'] );

                $fields_state = $this->validate_form_inputs( $_POST );

                $invalid_fields = array_filter( $fields_state, function ( $v, $k ) {
                    return ! $v;
                }, ARRAY_FILTER_USE_BOTH );

                if ( empty( $invalid_fields ) ) {
                    foreach ( $fields_state as $field => $state ) {
                        if ( $state ) {
                            $value = $this->prepare_input( $field, $_POST[ $field ] );

                            if ( get_option( $field ) !== false ) {
                                update_option( $field, $value );
                            } else {
                                add_option( $field, $value, '', 'yes' );
                            }
                        }
                    }

                    if ( array_key_exists( 'emkp_api_url', $_POST ) ) {
                        try {
                            $this->import_vat_rate();
                            $this->import_courier_accounts();
                        }catch (\Zitec\MkpApiWrapper\Exceptions\FailedRequestException $exception) {
                            $message = json_decode($exception->getMessage());

                            return [
                                'api' => $message,
                            ];
                        }
                    }
                }
            }

            return array_keys( $invalid_fields );
        }

        public function page() {
            global $wpdb;

            $emkp_order_statuses   = $this->emkp_order_statuses;
            $emkp_shipping_methods = $this->emkp_shipping_methods;

            $invalid_fields = $this->save_emkp_settings_data();

            $emkp_active_tab = isset( $_GET['tab'] ) ? $_GET['tab'] : 'api_config';
            if ( isset( $_GET['tab'] ) ) {
                $emkp_active_tab = wp_strip_all_tags( $_GET['tab'] );
            }

            $locales    = $this->locales;
            $currencies = $this->currencies;
            $localities = (new Emkp_Localities_Handler())->get_localities();

            $vat_rates = $wpdb->get_results( "SELECT * FROM {$wpdb->prefix}emkp_vat_rates ORDER BY is_default DESC, vat_rate ASC", ARRAY_A );

            $payment_methods = [];
            /** @var WC_Payment_Gateway $gateway */
            foreach ( WC_Payment_Gateways::instance()->get_available_payment_gateways() as $gateway ) {
                $payment_methods[ $gateway->id ] = $gateway->get_method_title();
            }

            $locality_id       = get_option( 'emkp_shipping_locality_id', null );
            $selected_locality = null;
            if ( $locality_id ) {
                $selected_locality = ( new Emkp_Localities_Handler() )->get_locality( $locality_id );
            }

            require_once( EMKP_ABSPATH . 'templates/settings/general.php' );
        }

        /**
         * Validate submitted data
         *
         * @param array $form_inputs
         *
         * @return array
         */
        private function validate_form_inputs( array $form_inputs ) {
            // required field validation logic
            $required_field = function ( $value ) {
                return strlen( $value ) !== false;
            };

            // email validation logic
            $validate_email = function ( $value ) {
                $email_array = explode( ',', $value );

                $is_valid = true;
                foreach ( $email_array as $email_key => $email ) {
                    $is_valid = $is_valid && ( filter_var( $email, FILTER_VALIDATE_EMAIL ) !== false );
                }

                return $is_valid;
            };

            $validations = [
                // API
                'emkp_api_url'                => [ $required_field, [ 'filter_var', FILTER_VALIDATE_URL ] ],
                'emkp_username'               => [ $required_field ],
                'emkp_password'               => [ $required_field ],
                'emkp_locale'                 => [ [ 'in_array', array_keys( $this->locales ) ] ],
                'emkp_currency'               => [ [ 'in_array', $this->currencies ] ],

                // PRODUCTS
                'emkp_product_queue_limit'    => [
                    $required_field,
                    [ 'filter_var', FILTER_VALIDATE_INT, [ 'options' => [ 'min_range' => 1 ] ] ]
                ],
                'emkp_halding_time'           => [
                    $required_field,
                    [ 'filter_var', FILTER_VALIDATE_INT, [ 'options' => [ 'min_range' => 1 ] ] ]
                ],
                'emkp_vat_rate'               => [
                    $required_field,
                    [ 'filter_var', FILTER_VALIDATE_INT, [ 'options' => [ 'min_range' => 0, 'max_range' => 100 ] ] ]
                ],
                'emkp_min_sale_price'         => [ $required_field, [ 'filter_var', FILTER_VALIDATE_FLOAT ], ],
                'emkp_max_sale_price'         => [ $required_field, [ 'filter_var', FILTER_VALIDATE_FLOAT ], ],

                // ORDERS
                'emkp_initial_order_status'   => [ [ 'in_array', array_keys( $this->emkp_order_statuses ) ] ],
                'emkp_finalized_order_status' => [ [ 'in_array', array_keys( $this->emkp_order_statuses ) ] ],
                'emkp_cancelled_order_status' => [ [ 'in_array', array_keys( $this->emkp_order_statuses ) ] ],

                // ALERTS
                'emkp_api_error_email'        => [ $required_field, $validate_email ],
                'emkp_import_order_email'     => [ $required_field, $validate_email ],

                // SHIPPING
                'emkp_shipping_name'          => [ $required_field, ],
                'emkp_shipping_contact'       => [ $required_field, ],
                'emkp_shipping_phone1'        => [ $required_field, [ 'filter_var', FILTER_SANITIZE_NUMBER_INT ] ],
                'emkp_shipping_locality'      => [ $required_field, [ 'in_array', array_keys( (new Emkp_Localities_Handler())->get_localities() ) ] ],
                'emkp_shipping_street'        => [ $required_field, ],
            ];

            $fields_state = [];

            foreach ( $form_inputs as $form_input => $value ) {
                $is_valid = true;

                if ( array_key_exists( $form_input, $validations ) ) {
                    $conditions = $validations[ $form_input ];

                    if ( ! is_array( $conditions ) ) {
                        $conditions = [ $conditions ];
                    }

                    foreach ( $conditions as $condition ) {
                        if ( is_array( $condition ) ) {
                            $function_name = array_shift( $condition );

                            // Add field value on the first position in the array
                            array_unshift( $condition, $value );

                            $is_valid = $is_valid && call_user_func_array( $function_name, $condition );
                        } elseif ( is_callable( $condition ) ) {
                            $is_valid = $is_valid && ( $condition( $value ) !== false );
                        }
                    }
                }

                $fields_state[ $form_input ] = $is_valid;
            }

            return $fields_state;
        }

        /**
         * Sanitize & format input values
         *
         * @param string $input
         * @param string $value
         *
         * @return string
         */
        private function prepare_input( $input, $value ) {
            $sanitize_emails = function ( $value ) {
                $emails = explode( ',', $value );

                $sanitized_emails = [];
                foreach ( $emails as $email ) {
                    $sanitized_emails[] = filter_var( trim( $email ), FILTER_SANITIZE_EMAIL );
                }

                return implode( ',', $sanitized_emails );
            };

            $preps = [
                'emkp_api_url'            => function ( $value ) {
                    return esc_url_raw( trim( $value ) );
                },
                'emkp_api_error_email'    => $sanitize_emails,
                'emkp_import_order_email' => $sanitize_emails,
            ];

            if ( array_key_exists( $input, $preps ) ) {
                return $preps[ $input ]( $value );
            }

            return sanitize_text_field( $value );
        }

        /**
         * @return bool|void
         *
         * @throws \Zitec\MkpApiWrapper\Exceptions\FailedRequestException
         */
        private function import_vat_rate() {
            global $wpdb;

            $client = $this->get_client_with_post_data();
            if ( ! $client ) {
                return;
            }

            $request = new \Zitec\MkpApiWrapper\Requests\Vat\ReadRequest();

            $response = $client->sendRequest( $request );

            if ( $response->isError ) {
                throw new \Zitec\MkpApiWrapper\Exceptions\FailedRequestException( json_encode( $response->messages ) );
            }

            if ( ! isset( $response->results ) ) {
                return false;
            }

            $existing_vat_rates_ids = $wpdb->get_col( "SELECT vat_id FROM {$wpdb->prefix}emkp_vat_rates" );

            $vat_rates      = [];
            $default_option = 0;

            foreach ( $response->results as $result ) {
                if ( ! is_object( $result ) || ! property_exists( $result, 'vat_id' ) || ! property_exists( $result, 'vat_rate' ) || ! property_exists( $result, 'is_default' ) ) {
                    continue;
                }

                $properties = [
                    sprintf( '%d', $result->vat_id ),
                    sprintf( '%F', $result->vat_rate ),
                    $result->is_default ?: 0
                ];

                $vat_rates[ $result->vat_id ] = implode( ',', $properties );

                if ( $result->is_default ) {
                    $default_option = $result->vat_id;
                }
            }

            $deleted_vat_rates = array_diff( $existing_vat_rates_ids, array_keys( $vat_rates ) );

            $values = array_map( function ( $row ) {
                return '(' . $row . ')';
            }, $vat_rates );

            if ( ! empty( $values ) ) {
                $wpdb->query( "INSERT INTO {$wpdb->prefix}emkp_vat_rates (vat_id, vat_rate, is_default) VALUES " . implode( ',', $values ) . "
                ON DUPLICATE KEY UPDATE vat_rate = VALUES(vat_rate), is_default = VALUES(is_default)" );
            }

            if ( ! empty( $deleted_vat_rates ) ) {
                $wpdb->query( "DELETE FROM {$wpdb->prefix}emkp_vat_rates WHERE vat_id IN (" . implode( ',', $deleted_vat_rates ) . ")" );
            }

            if ( get_option( 'emkp_vat_rate' ) === false && $default_option ) {
                add_option( 'emkp_vat_rate', $default_option, '', 'yes' );
            }
        }

        /**
         * @return \Zitec\MkpApiWrapper\InstantiableClient|null
         */
        private function get_client_with_post_data() {
            /** @var Emkp $emkp */
            global $emkp;
            $api_url      = esc_url_raw( get_option( 'emkp_api_url' ) ?: $_POST['emkp_api_url'] );
            $api_username = esc_html( get_option( 'emkp_username' ) ?: $_POST['emkp_username'] );
            $api_password = esc_html( get_option( 'emkp_password' ) ?: $_POST['emkp_password'] );

            if ( ! $api_url || ! $api_username || ! $api_password ) {
                return null;
            }

            return $emkp->get_client_instance( $api_url, $api_username, $api_password );
        }

        /**
         * @return bool|void
         *
         * @throws \Zitec\MkpApiWrapper\Exceptions\FailedRequestException
         */
        private function import_courier_accounts() {
            global $wpdb;

            $client = $this->get_client_with_post_data();
            if ( ! $client ) {
                return;
            }

            $request  = new Zitec\MkpApiWrapper\Requests\CourierAccount\ReadRequest();
            $response = $client->sendRequest( $request );

            if ( $response->isError ) {
                throw new \Zitec\MkpApiWrapper\Exceptions\FailedRequestException( json_encode( $response->messages ) );
            }

            if ( ! isset( $response->results ) ) {
                return false;
            }

            $emag_couriers = $response->results;

            $existing_couriers = $wpdb->get_col( "SELECT emag_id FROM {$wpdb->prefix}emkp_couriers" );

            $couriers = [];

            foreach ( $emag_couriers as $courier ) {
                if (
                    ! $courier->status ||
                    ! is_object( $courier ) ||
                    ! property_exists( $courier, 'account_id' ) ||
                    ! property_exists( $courier, 'account_display_name' ) ||
                    ! property_exists( $courier, 'courier_name' )
                ) {
                    continue;
                }

                $couriers[ $courier->account_id ] = implode( ',', [
                    $courier->account_id,
                    "'" . $courier->account_display_name . "'",
                    "'" . $courier->courier_name . "'",
                ] );
            }

            $deleted_couriers = array_diff( $existing_couriers, array_keys( $couriers ) );

            $values = array_map( function ( $row ) {
                return '(' . $row . ')';
            }, $couriers );

            if ( ! empty( $values ) ) {
                $wpdb->query( "INSERT INTO {$wpdb->prefix}emkp_couriers (emag_id, display_name, name) VALUES " . implode( ',', $values ) . "
                ON DUPLICATE KEY UPDATE name = VALUES(name), display_name = VALUES(display_name)" );
            }

            if ( ! empty( $deleted_couriers ) ) {
                $wpdb->query( "DELETE FROM {$wpdb->prefix}emkp_couriers WHERE emag_id IN (" . implode( ',', $deleted_couriers ) . ")" );
            }
        }

        public function sanitize_emails( $input ) {
            $emails = explode( ',', $input );

            $sanitized_emails = [];
            foreach ( $emails as $email ) {
                $sanitized_emails[] = filter_var( trim( $email ), FILTER_SANITIZE_EMAIL );
            }

            return implode( ',', $sanitized_emails );
        }
    }

endif;
