<?php

global $wpdb;

if ( ! class_exists( 'Emkp_Order_Update' ) ) :

    /**
     * Class Emkp_Order_Update
     */
    class Emkp_Order_Update {

        /**
         * Order Statuses Mapping array
         *
         * @var array
         */
        private $emkp_statuses = [];

        /**
         * Emkp_Order_Update constructor
         */
        public function __construct() {
            // Init Statuses Mapping
            $this->init_emkp_statuses();

            // Add update action hooks
            $this->add_hooks();
        }

        /**
         * Initialize Statuses Mapping
         */
        private function init_emkp_statuses() {
            $status_initial   = get_option( 'emkp_initial_order_status' );
            $status_finalized = get_option( 'emkp_finalized_order_status' );
            $status_cancelled = get_option( 'emkp_cancelled_order_status' );

            $status_initial   = 'wc-' === substr( $status_initial, 0, 3 ) ? substr( $status_initial, 3 ) : $status_initial;
            $status_finalized = 'wc-' === substr( $status_finalized, 0, 3 ) ? substr( $status_finalized, 3 ) : $status_finalized;
            $status_cancelled = 'wc-' === substr( $status_cancelled, 0, 3 ) ? substr( $status_cancelled, 3 ) : $status_cancelled;

            $this->emkp_statuses['pending']           = 1;
            $this->emkp_statuses['processing']        = 2;
            $this->emkp_statuses['on-hold']           = 3;
            $this->emkp_statuses['completed']         = 4;
            $this->emkp_statuses['cancelled']         = 0;
            $this->emkp_statuses['refunded']          = 5;
            $this->emkp_statuses[ $status_initial ]   = 1;
            $this->emkp_statuses[ $status_finalized ] = 4;
            $this->emkp_statuses[ $status_cancelled ] = 0;
        }

        /**
         * Add action hooks for order updates
         */
        private function add_hooks() {
            add_action( 'pre_post_update', array( $this, 'before_order_update' ), 10, 2 );
            add_action( 'woocommerce_order_status_changed', array( $this, 'status_changed' ) );
            add_action( 'woocommerce_saved_order_items', array( $this, 'saved_order_items' ) );
            add_action( 'woocommerce_before_order_object_save', array( $this, 'calculate_totals' ) );
        }

        /**
         * Update Emag order on status changed in Woocommerce
         *
         * @param int $order_id
         */
        public function status_changed( $order_id ) {
            global $wpdb;

            // Load order from id
            $order = wc_get_order( $order_id );

            // Stop here if order id not found
            if ( ! $order ) {
                return;
            }

            // Load emkp order line from Woocomerce order id
            $emkp_order = $this->get_emkp_order_by_wc_id( $order_id );

            // Stop here if Emkp order line not found
            if ( ! $emkp_order ) {
                return;
            }

            // Check if new status is not initial order status set in Emkp settings, and new status is mapped
            $emkp_initial_order_status = get_option( 'emkp_initial_order_status' );
            $emkp_initial_order_status = 'wc-' === substr( $emkp_initial_order_status, 0, 3 ) ? substr( $emkp_initial_order_status, 3 ) : $emkp_initial_order_status;
            $new_status                = $order->get_status();

            if ( $emkp_initial_order_status == $new_status || ! isset( $emkp_order ) ) {
                return;
            }

            if ( ! array_key_exists( $new_status, $this->emkp_statuses ) ) {
                return;
            }

            // Decode Emag order response and set new status
            $response           = json_decode( $emkp_order['response'], true );
            $response['status'] = $this->emkp_statuses[ $new_status ];

            // Encode response with new status and save it
            $response_string = json_encode( $response );
            $tablename       = $wpdb->prefix . 'emkp_orders_queue';

            $wpdb->update( $tablename, array(
                'response' => $response_string,
                'status'   => 0,
            ), array( 'id' => $emkp_order['id'] ) );

            //  Send update request to Emag
            $request = new \Zitec\MkpApiWrapper\Requests\Order\SaveRequest( [
                    [
                        'id'     => $emkp_order['emag_order_id'],
                        'status' => $this->emkp_statuses[ $new_status ]
                    ]
                ]
            );

            $response = \Zitec\MkpApiWrapper\Client::sendRequest( $request );

            // Show error in admin if present
            if ( ! is_object( $response ) || ! property_exists( $response, 'isError' ) || $response->isError ) {
                if ( property_exists( $response, 'messages' ) && is_array( $response->messages ) ) {
                    $message = implode( ', ', $response->messages );
                } else {
                    $message = $response;
                }
                echo '<div class="error"><p>' . $message . '</p></div>';
            }
        }

        /**
         * Update Emag order on items save
         *
         * @param int $order_id
         */
        public function saved_order_items( $order_id ) {
            global $wpdb;

            // Load order from id
            $order = wc_get_order( $order_id );

            // Stop here if order id not found
            if ( ! $order ) {
                return;
            }

            // Load emkp order line from Woocomerce order id
            $emkp_order = $this->get_emkp_order_by_wc_id( $order_id );

            // Stop here if Emkp order line not found
            if ( ! $emkp_order ) {
                return;
            }

            // Force order total calculation, for taxes
            $order->calculate_totals();

            // Get order items
            $order_items = $order->get_items();

            // Get Emkp response and clear products array
            $emkp_order_response             = json_decode( $emkp_order['response'], true );
            $emkp_order_response['products'] = array();

            // Get Emkp vat rate
            $vat_rate = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM {$wpdb->prefix}emkp_vat_rates WHERE vat_id = %d", [ get_option( 'emkp_vat_rate' ) ] ) );

            // Create order items and add to products array
            foreach ( $order_items as $item ) {
                $item_price = $order->get_item_total( $item );
                $sale_price = $item_price;

                $emkp_order_response['products'][] = array(
                    'product_id' => $item['product_id'],
                    'sale_price' => $sale_price,
                    'quantity'   => $item['quantity'],
                    'status'     => 1,
                    'vat'        => $vat_rate->vat_rate
                );
            }

            // Update shipping total
            $shipping_total = 0;
            foreach ( $order->get_shipping_methods() as $shipping ) {
                $shipping_total += $shipping->get_total() + $shipping->get_total_tax();
            }

            $emkp_order_response['shipping_tax'] = $shipping_total;

            // Save response data
            $response_string = json_encode( $emkp_order_response );
            $tablename       = $wpdb->prefix . 'emkp_orders_queue';

            $wpdb->update( $tablename, array(
                'response' => $response_string,
                'status'   => 0,
            ), array( 'id' => $emkp_order['id'] ) );

            // Send Request
            $request = new \Zitec\MkpApiWrapper\Requests\Order\SaveRequest( [
                    $emkp_order_response
                ]
            );

            $response = \Zitec\MkpApiWrapper\Client::sendRequest( $request );

            // Show error in admin if present
            if ( ! is_object( $response ) || ! property_exists( $response, 'isError' ) || $response->isError ) {
                if ( property_exists( $response, 'messages' ) && is_array( $response->messages ) ) {
                    $message = implode( ', ', $response->messages );
                } else {
                    $message = $response;
                }
                echo '<div class="error"><p>' . $message . '</p></div>';
            }
        }

        /**
         * Get Emkp order line using Woocommerce order id
         *
         * @param int $order_id
         *
         * @return mixed
         */
        public function get_emkp_order_by_wc_id( $order_id ) {
            global $wpdb;

            $order = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM {$wpdb->prefix}emkp_orders_queue WHERE wc_order_id = %d LIMIT 1 ", [ $order_id ] ), ARRAY_A );

            return ( ! is_null( $order ) ) ? $order : false;
        }

        /**
         * @param int $payment_mode_id
         *
         * @return mixed
         */
        private function get_payment_method( $payment_mode_id ) {
            $payment_mode_mapping = [
                1 => 'cod',
                2 => 'bank_transfer',
                3 => 'card'
            ];

            if ( ! array_key_exists( $payment_mode_id, $payment_mode_mapping ) ) {
                return false;
            }

            return get_option( 'emkp_payment_' . $payment_mode_mapping[ $payment_mode_id ] );
        }

        /**
         * @param WC_Order $order
         *
         */
        public function calculate_totals( WC_Order $order ) {
            if ( ! $order ) {
                return;
            }

            // Load emkp order line from Woocomerce order id
            $emkp_order = $this->get_emkp_order_by_wc_id( $order->get_id() );

            // Stop here if Emkp order line not found
            if ( ! $emkp_order ) {
                return;
            }

            $emkp_order_std = json_decode( $emkp_order['response'] );

            //Set discount total
            $discount_total = 0;
            $order_total    = $order->get_total();

            if ( $emkp_order_std->vouchers && count( $emkp_order_std->vouchers ) > 0 ) {
                foreach ( $emkp_order_std->vouchers as $voucher ) {
                    $discount_total += $voucher->sale_price + $voucher->sale_price_vat;
                }
            }

            $order_total = $order_total - $order->get_discount_total() + $discount_total;
            $order->set_discount_total( $discount_total );

            // set payment type and title
            $order->set_payment_method( $this->get_payment_method( $emkp_order_std->payment_mode_id ) );
            $order->set_payment_method_title( $emkp_order_std->payment_mode );

            // Calculate and save total
            $grand_total = round( $order_total, wc_get_price_decimals() );
            $order->set_total( $grand_total );
        }

        /**
         * Verify if status transition is allowed in Emag
         *
         * @param int $post_ID
         * @param array $data
         *
         */
        public function before_order_update( $post_ID, $data ) {
            // Verify if information is present and update is on an order
            if ( php_sapi_name() === 'cli' || $data['post_type'] !== 'shop_order' || $_SERVER['REQUEST_METHOD'] !== 'POST' || ! isset( $_POST ) ) {
                return;
            }

            // Load emkp order line from Woocomerce order id
            $emkp_order = $this->get_emkp_order_by_wc_id( $post_ID );

            // Stop here if Emkp order line not found
            if ( ! $emkp_order ) {
                return;
            }

            // Check status matrix
            $current_status            = $data['post_status'];
            $new_status                = esc_html( $_POST['order_status'] );
            $status_transition_allowed = $this->is_status_change_allowed( $current_status, $new_status );

            // If not status transition not allowed, stop update and show error message
            if ( ! $status_transition_allowed ) {
                wp_die( __( 'Status transition not allowed in Emag', 'emkp' ), __( 'Emag status change verification', 'emkp' ), array( 'back_link' => true ) );
            }
        }

        /**
         * Verify Emag Order Status Matrix
         *
         * @param string $current_status
         * @param string $new_status
         *
         * @return mixed
         */
        private function is_status_change_allowed( $current_status, $new_status ) {
            $current_status = 'wc-' === substr( $current_status, 0, 3 ) ? substr( $current_status, 3 ) : $current_status;
            $new_status     = 'wc-' === substr( $new_status, 0, 3 ) ? substr( $new_status, 3 ) : $new_status;

            if ( ! array_key_exists( $current_status, $this->emkp_statuses ) || ! key_exists( $new_status, $this->emkp_statuses ) ) {
                return;
            }

            $current_status = $this->emkp_statuses[ $current_status ];
            $new_status     = $this->emkp_statuses[ $new_status ];

            // Status change matrix
            $status_matrix = [
                1 => [ 1, 2 ],
                2 => [ 2, 3, 4, 0 ],
                3 => [ 3, 4, 0 ],
                4 => [ 3, 4, 0, 5 ],
                0 => [ 2, 3, 4, 0 ],
                5 => [ 5 ]
            ];

            return ( in_array( $new_status, $status_matrix[ $current_status ] ) );
        }
    }

    $emkp_orders = new Emkp_Order_Update();

endif;
