<?php
/**
 * Class EMKP_Orders
 */
if ( ! class_exists( 'EMKP_Orders' ) ):
    class EMKP_Orders {

        /**
         * Import Emag orders and save/update in Woocommerce orders
         *
         */
        public function import_emag_orders() {
            global $wpdb;

            // Get all unsynchronised emag orders
            $order_ids = $wpdb->get_results( "SELECT * FROM {$wpdb->prefix}emkp_orders_queue WHERE status = 0", ARRAY_A );

            // Stop here if no orders found
            if ( ! $order_ids || count( $order_ids ) <= 0 ) {
                return;
            }

            $alert = new Emkp_Email_Alert_Manager();
            $tablename       = $wpdb->prefix . 'emkp_orders_queue';
            // Import each order
            foreach ( $order_ids as $order_id ) {
                $response = $this->do_import_emag_orders( $order_id['emag_order_id'] );

                // Stop here if error or no results
                if ( ! $response || $response->isError || ! $response->results || count( $response->results ) <= 0 ) {

                    $message = 'Response error or missing. ' . json_encode( (array) $response );

                    $wpdb->update( $tablename, array(
                        'status'      => '1',
                        'response'    => $message,
                    ), array( 'id' => $order_id['id'] ) );

                    $alert->alert( 'emag_error', $message );
                    
                    continue;
                }

                // Save imported order
                foreach ( $response->results as $order ) {
                    if ( $order && count( $order ) > 0 ) {
                        $this->save_order( $order, $order_id['id'] );
                    }
                }
            }
        }

        /**
         * Save order results
         *
         * @param stdClass $order
         * @param int      $ack_id
         */
        private function save_order( $order, $ack_id ) {
            global $wpdb;

            // Check if all Emkp products exist in Woocommerce , if not send error email
            // Stop here, do not create order.
            $wc_products_list = array();
            
            if ( $order->products && count( $order->products ) > 0 ) {
                foreach ( $order->products as $product ) {
                    // Skip if product has been removed from order
                    if ( $product->status !== 1 ) {
                        continue;
                    }

                    // Load WooCommerce product from Emag product_id
                    $_pf        = new WC_Product_Factory();
                    $wc_product = $_pf->get_product( $product->product_id );

                    // If product not found, stop here and send alert email
                    if ( $wc_product === false ) {
                        $alert = new Emkp_Email_Alert_Manager();
                        $message = __( 'Invalid product id received ' ) . $product->product_id . '<br />' . print_r( $order, true );
                        $response_string = json_encode( (array) $order );
                        $tablename       = $wpdb->prefix . 'emkp_orders_queue';

                        $wpdb->update( $tablename, array(
                            'status'      => '1',
                            'response'    => $response_string
                        ), array( 'id' => $ack_id ) );

                        $alert->alert( 'emag_error', $message );
                        return;
                    }

                    // Create product data to be added to new order, with price from Emag
                    $sale_price  = $product->sale_price;
                    $price_total = round( $sale_price * $product->quantity, wc_get_price_decimals() );

                    $default_args = array(
                        'product_id' => $product->product_id,
                        'subtotal'   => $price_total,
                        'total'      => $price_total,
                        'quantity'   => $product->quantity,
                    );

                    $wc_products_list[] = array(
                        'product'  => $wc_product,
                        'quantity' => $product->quantity,
                        'args'     => $default_args
                    );
                }
            }

            // Check if emkp order already has been read and assigned to a woocommerce order
            $result = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM {$wpdb->prefix}emkp_orders_queue WHERE emag_order_id=%d AND wc_order_id IS NOT NULL", [ $order->id ] ) );

            if ( ! is_null( $result ) ) {
                // Load existing order
                $wc_order = wc_create_order( array( 'order_id' => $result->wc_order_id ) );

                // Clear old products
                $wc_order->remove_order_items();

                // Set flag order is not new
                $new_order = false;
            } else {
                // Create the order
                $wc_order = wc_create_order();

                // Set flag order is new
                $new_order = true;

                // Set order created date from Emag created date.
                $wc_order->set_date_created( $order->date );
            }

            // Set order status , default and override with mapping from Emkp settings
            $wc_status_code = array(
                1 => get_option( 'emkp_initial_order_status' ) ?: 'wc-pending',
                2 => 'wc-processing',
                3 => 'wc-on-hold',
                4 => get_option( 'emkp_finalized_order_status' ) ?: 'wc-completed',
                0 => get_option( 'emkp_cancelled_order_status' ) ?: 'wc-cancelled',
                5 => 'refunded'
            );

            // Set status
            $wc_order->update_status( $wc_status_code[ $order->status ] );

            // Add products to order
            foreach ( $wc_products_list as $wc_prod ) {
                $wc_order->add_product( $wc_prod['product'], $wc_prod['quantity'], $wc_prod['args'] );
            }

            // Get addresses information from Emkp response, map to Woocommerce structure
            $billing_address  = $this->get_emkp_billing_address( $order );
            $shipping_address = $this->get_emkp_shipping_address( $order );

            // Set billing and shipping adresses to order
            $wc_order->set_address( $billing_address, 'billing' );
            $wc_order->set_address( $shipping_address, 'shipping' );

            // If shipping tax exists, create order shipping line item and add to order
            if ( $order->shipping_tax && $order->shipping_tax > 0 ) {
                $shipping_item = $this->get_emkp_shipping_line_item( $wc_order, $order->shipping_tax );
                $wc_order->add_item( $shipping_item );
            }

            // Add Observations to the order
            if ( $order->observation && ! is_null( $order->observation ) ) {
                $wc_order->add_order_note( $order->observation );
            }

            // Save original order details and update status, flag order as synchronised
            $response_string = json_encode( (array) $order );
            $tablename       = $wpdb->prefix . 'emkp_orders_queue';

            $wpdb->update( $tablename, array(
                'status'      => '1',
                'wc_order_id' => $wc_order->get_id(),
                'response'    => $response_string
            ), array( 'id' => $ack_id ) );

            // Calculate totals, and add to order
            $wc_order->calculate_totals();

            // Save order object
            $wc_order->save();

            // If new order
            // Send acknowledge request back to Emkp
            if ( $new_order ) {
                $request = new \Zitec\MkpApiWrapper\Requests\Order\AcknowledgeRequest( [
                    'order_id' => $order->id
                ], true );

                \Zitec\MkpApiWrapper\Client::sendRequest( $request );
            }
        }

        /**
         * Create WC_Order_Item_Shipping from emag shipping tax
         *
         * @param WC_Order $wc_order
         * @param float    $emkp_shipping_total
         *
         * @return WC_Order_Item_Shipping
         */
        private function get_emkp_shipping_line_item( WC_Order $wc_order, $emkp_shipping_total ) {
            $shipping = new WC_Order_Item_Shipping();

            // Get tax rates for shipping
            if ( wc_tax_enabled() ) {

                $args         = array();
                $tax_based_on = get_option( 'woocommerce_tax_based_on' );
                $args         = wp_parse_args( $args, array(
                    'country'  => 'billing' === $tax_based_on ? $wc_order->get_billing_country() : $wc_order->get_shipping_country(),
                    'state'    => 'billing' === $tax_based_on ? $wc_order->get_billing_state() : $wc_order->get_shipping_state(),
                    'postcode' => 'billing' === $tax_based_on ? $wc_order->get_billing_postcode() : $wc_order->get_shipping_postcode(),
                    'city'     => 'billing' === $tax_based_on ? $wc_order->get_billing_city() : $wc_order->get_shipping_city(),
                ) );

                $shipping_tax_class = get_option( 'woocommerce_shipping_tax_class' );

                // Inherit tax class from items
                if ( 'inherit' === $shipping_tax_class ) {
                    $tax_rates         = array();
                    $tax_classes       = array_merge( array( '' ), WC_Tax::get_tax_class_slugs() );
                    $found_tax_classes = $wc_order->get_items_tax_classes();

                    foreach ( $tax_classes as $tax_class ) {
                        if ( in_array( $tax_class, $found_tax_classes ) ) {
                            $tax_rates = WC_Tax::find_shipping_rates( array(
                                'country'   => $args['country'],
                                'state'     => $args['state'],
                                'postcode'  => $args['postcode'],
                                'city'      => $args['city'],
                                'tax_class' => $tax_class,
                            ) );
                            break;
                        }
                    }
                } else {
                    $tax_rates = WC_Tax::find_shipping_rates( array(
                        'country'   => $args['country'],
                        'state'     => $args['state'],
                        'postcode'  => $args['postcode'],
                        'city'      => $args['city'],
                        'tax_class' => $shipping_tax_class,
                    ) );
                }

                // Set shipping total and taxes from Emag shipping tax
                $shipping->set_taxes( array( 'total' => WC_Tax::calc_tax( $emkp_shipping_total, $tax_rates, true ) ) );
                $total = $emkp_shipping_total - $shipping->get_total_tax();
                $shipping->set_total( $total );

                return $shipping;
            } else {
                // If taxes not enabled, nothing to calculate, set shipping total as received from Emag
                $shipping->set_total( $emkp_shipping_total );

                return $shipping;
            }
        }

        /**
         * Create array with address information to be saved in Woocomerce
         *
         * @param stdClass $order
         *
         * @return array
         */
        private function get_emkp_billing_address( $order ) {
            $billing_address = array();

            if ( $order->customer ) {
                // Name comes in one single field, assume lastname is first
                $name = explode( ' ', $order->customer->name );
                if ( key_exists( 0, $name ) ) {
                    $lastname = $name[0];
                }
                unset( $name[0] );
                $firstname = join( ' ', $name );

                // Construct address data
                $billing_address['first_name'] = $firstname;
                $billing_address['last_name']  = $lastname;

                $billing_address['company']   = $order->customer->company;
                $billing_address['phone']     = $order->customer->phone_1;
                $billing_address['email']     = $order->customer->email;
                $billing_address['address_1'] = $order->customer->billing_street;
                $billing_address['city']      = $order->customer->billing_city;
                $billing_address['state']     = $order->customer->billing_suburb;
                $billing_address['country']   = $order->customer->billing_country;
                $billing_address['postcode']  = $order->customer->billing_postal_code;
            }

            return $billing_address;
        }

        /**
         * Create array with address information to be saved in Woocomerce
         *
         * @param stdClass $order
         *
         * @return array
         */
        private function get_emkp_shipping_address( $order ) {
            $shipping_address = array();

            if ( $order->customer ) {
                $shipping_name = explode( ' ', $order->customer->shipping_contact );
                if ( key_exists( 0, $shipping_name ) ) {
                    $shipping_lastname = $shipping_name[0];
                }
                unset( $shipping_name[0] );
                $shipping_firstname             = join( ' ', $shipping_name );
                $shipping_address['first_name'] = $shipping_firstname;
                $shipping_address['last_name']  = $shipping_lastname;

                $shipping_address['phone']     = $order->customer->shipping_phone;
                $shipping_address['email']     = $order->customer->email;
                $shipping_address['address_1'] = $order->customer->shipping_street;
                $shipping_address['city']      = $order->customer->shipping_city;
                $shipping_address['state']     = $order->customer->shipping_suburb;
                $shipping_address['country']   = $order->customer->shipping_country;
                $shipping_address['postcode']  = $order->customer->shipping_postal_code;
            }

            return $shipping_address;
        }

        /**
         * Read Emag order request
         *
         * @param int $order_id
         *
         * @return array
         */
        private function do_import_emag_orders( $order_id ) {

            $request = new \Zitec\MkpApiWrapper\Requests\Order\ReadRequest( [ 'id' => $order_id ] );

            return \Zitec\MkpApiWrapper\Client::sendRequest( $request );
        }

    }
endif;

return [
    'command_name' => 'import-emkp-orders',
    'callback'     => function () {
        $emkp_orders = new EMKP_Orders();

        try {
            $emkp_orders->import_emag_orders();

        } catch ( \Throwable  $exception ) {
            $alert = new \Emkp_Email_Alert_Manager();
            $alert->alert( 'emag_error', print_r( $exception, true ) );
        }
    },
];
