<?php

/**
 * Class Emkp_Product_Sender
 */
if ( ! class_exists( 'Emkp_Product_Sender' ) ):
    class Emkp_Product_Sender {
        /**
         * @var null|WC_Product
         */
        private $product;

        /**
         * @var wpdb wpdb
         */
        private $wpdb;

        /**
         * @var int
         */
        private $emag_category_id;

        /**
         * Products array
         *
         * @var array
         */
        private $products = [];

        /**
         * @var int
         */
        private $products_sent = 0;

        /**
         * @var string deleted|updated
         */
        private $state;

        /**
         * Emkp_Product_Sender constructor.
         *
         * @param int    $product_id
         * @param string $state
         * @param int    $products_sent
         *
         * @throws Exception
         */
        public function __construct( $product_id, $state, &$products_sent ) {
            global $wpdb;

            $this->wpdb    = $wpdb;
            $this->product = wc_get_product( $product_id );
            $this->state   = $state;

            if ( ! $this->product ) {
                $this->product = $product_id;
                $this->state   = 'deleted';
            } elseif ( $this->product && ! ( $this->product instanceof WC_Product_Simple || $this->product instanceof WC_Product_Variable ) ) {
                throw new Exception( 'Invalid product!' );
            }

            $this->prepare_data();

            $this->products_sent = $products_sent;
        }

        /**
         * @return array
         */
        public function send() {
            $request = new \Zitec\MkpApiWrapper\Requests\ProductOffer\CreateRequest( $this->products, true );

            $this->products_sent += count( $this->products );

            return \Zitec\MkpApiWrapper\Client::sendRequest( $request );
        }

        private function prepare_data() {
            if ( $this->product instanceof WC_Product_Simple ) {
                $data = $this->get_product_data( $this->product );

                if ( ! empty( $data ) ) {
                    $this->products[] = $data;
                }
            } elseif ( $this->product instanceof WC_Product_Variable ) {
                $page       = 1;
                $variations = [];
                do {
                    $_variations = wc_get_products( array(
                        'status'  => array( 'private', 'publish', 'trash' ),
                        'type'    => 'variation',
                        'parent'  => $this->product->get_id(),
                        'limit'   => 100,
                        'page'    => $page,
                        'orderby' => array(
                            'menu_order' => 'ASC',
                            'ID'         => 'DESC',
                        ),
                        'return'  => 'objects',
                    ) );
                    $page ++;

                    foreach ( $_variations as $variation ) {
                        $variations[] = $variation;
                    }
                } while ( ! empty( $_variations ) );

                foreach ( $variations as $variation ) {
                    if ( $variation && $variation instanceof WC_Product_Variation && $variation->get_manage_stock() === true ) {
                        $data = $this->get_product_data( $variation, true );

                        if ( ! empty( $data ) ) {
                            $this->products[] = $data;
                        }
                    }
                }
            } elseif ( is_numeric( $this->product ) && $this->state === 'deleted' ) {
                $this->products[] = [
                    'id'     => $this->product,
                    'status' => 0,
                ];
            }
        }

        /**
         * @param WC_Product $product
         * @param bool       $is_variation
         *
         * @return array
         */
        private function get_product_data( WC_Product $product, $is_variation = false ) {
            if ( $is_variation && ( $product->get_meta( '_emkp_visible_to_marketplace' ) !== 'yes' && get_post_meta( $product->get_id(), 'emkp_sent_to_emag', true ) !== 'yes' ) ) {
                return [];
            }

            if ( $this->state === 'deleted' ) {
                $status = 0;
            } elseif ( $product instanceof WC_Product_Simple ) {
                $status = (int) ( $product->get_status() === 'publish' && $product->get_meta( '_emkp_visible_to_marketplace' ) === 'yes' );
            } elseif ( $product instanceof WC_Product_Variation ) {
                $status = (int) ( $this->product->get_status() === 'publish' && $product->get_meta( '_emkp_visible_to_marketplace' ) === 'yes' );
            } else {
                $status = 0;
            }

            $emag_data = [
                'id'                => $product->get_id(),
                'part_number'       => $product->get_sku(),
                'url'               => get_post_permalink( $product->get_id() ),
                'status'            => $status,
                'recommended_price' => $this->get_product_price( $product, 'regular' ),
                'sale_price'        => $this->get_product_price( $product, 'sale' ),
                'min_sale_price'    => $this->get_product_price( $product, 'min' ),
                'max_sale_price'    => $this->get_product_price( $product, 'max' ),
                'availability'      => [
                    [
                        'warehouse_id' => 1,
                        'id'           => $this->get_product_availability( $product ),
                    ]
                ],
                'stock'             => [
                    [
                        'warehouse_id' => 1,
                        'value'        => $product->get_stock_quantity(),
                    ]
                ],
                'handling_time'     => [
                    [
                        'warehouse_id' => 1,
                        'value'        => get_option( 'emkp_handling_time' ),
                    ]
                ],
                'vat_id'            => get_option( 'emkp_vat_rate' ),
                'warranty'          => $this->product->get_meta( '_emkp_warranty' ) ?: 0,
                'name'              => $product->get_name()
            ];

            if ( $this->state !== 'deleted' ) {
                $emag_data['category_id']     = $this->get_product_emag_category();
                $emag_data['characteristics'] = $this->get_product_characteristics( $product );
            }

            if ( $product->get_meta( '_emkp_pnk' ) ) {
                $emag_data['part_number_key'] = $product->get_meta( '_emkp_pnk' );
            }

            $emag_data['description'] = $product->get_description();
            $emag_data['brand']       = $this->product->get_meta( '_emkp_brand' );
            $emag_data['images']      = $this->get_product_images( $product );
            $emag_data['family']      = [ 'id' => 0 ];

            // Override data if this is a variation
            if ( $is_variation ) {
                $emag_data['name']   = $this->get_name_of_variation( $product );
                $emag_data['status'] = $product->variation_is_visible() && $product->get_meta( '_emkp_visible_to_marketplace' ) === 'yes';

                if ( empty( $emag_data['description'] ) ) {
                    $emag_data['description'] = $this->product->get_description();
                }

                if ( empty( $emag_data['images'] ) ) {
                    $emag_data['images'] = $this->get_product_images( $this->product );
                }
            }

            $emag_data['status'] = (int) $emag_data['status'];

            update_post_meta( $product->get_id(), 'emkp_sent_to_emag', 'yes' );

            return $emag_data;
        }

        /**
         * @return int|mixed|null
         */
        private function get_product_emag_category() {
            if ( empty( $this->product->get_category_ids() ) ) {
                return null;
            }

            $args = array(
                'taxonomy'     => 'product_cat',
                'orderby'      => 'parent',
                'order'        => 'ASC',
                'show_count'   => false,
                'pad_counts'   => false,
                'hierarchical' => false,
                'title_li'     => '',
                'hide_empty'   => false,
                'include'      => $this->product->get_category_ids(),
            );

            $category_ids = [];
            foreach ( get_categories( $args ) as $category ) {
                $category_ids[] = $category->term_id;
            }

            if ( empty( $category_ids ) ) {
                return null;
            }

            $mapped_categories = $this->wpdb->get_col( "SELECT emag_category_id FROM {$this->wpdb->prefix}emkp_categories_match  WHERE woocommerce_category_id IN (" . implode( ',',
                    $category_ids ) . ")" );

            if ( ! $this->emag_category_id ) {
                $this->emag_category_id = array_shift( $mapped_categories );
            }

            return $this->emag_category_id;
        }

        /**
         * @param WC_Product $product
         *
         * @return array
         */
        private function get_product_images( WC_Product $product ) {
            $images = [];

            $main_image = wp_get_attachment_image_url( get_post_thumbnail_id( $product->get_id() ) );

            if ( $main_image ) {
                $images[] = [
                    'display_type' => 1,
                    'url'          => $main_image,
                ];
            }

            $attachmet_ids = $product->get_gallery_image_ids();

            foreach ( $attachmet_ids as $attachmet_id ) {
                $image_url = wp_get_attachment_url( $attachmet_id );

                if ( $image_url ) {
                    $images[] = [
                        'display_type' => 2,
                        'url'          => $image_url,
                    ];
                }
            }

            return $images;
        }

        /**
         * @param WC_Product $product
         * @param string     $price_type
         *
         * @return bool|string
         */
        private function get_product_price( WC_Product $product, $price_type ) {
            switch ( $price_type ) {
                case 'regular':
                    $product_price = $product->get_regular_price();
                    break;

                case 'sale':
                    $product_price = $product->get_sale_price();

                    if ( ! $product_price || ! $product->is_on_sale() ) {
                        $product_price = $product->get_regular_price();
                    }
                    break;

                case 'min':
                    return (double) number_format( $this->get_product_price( $product, 'sale' ) * ( 1 - get_option( 'emkp_min_sale_price' ) / 100 ), 4, '.', '' );
                    break;

                case 'max':
                    return (double) number_format( $this->get_product_price( $product, 'sale' ) * ( 1 + get_option( 'emkp_max_sale_price' ) / 100 ), 4, '.', '' );
                    break;

                default:
                    return false;
                    break;
            }

            $price_include_tax = get_option( 'woocommerce_prices_include_tax' );

            if ( $price_include_tax === 'yes' ) {
                $vat_rate = $this->wpdb->get_row( $this->wpdb->prepare( "SELECT * FROM {$this->wpdb->prefix}emkp_vat_rates WHERE vat_id = %d",
                    [ get_option( 'emkp_vat_rate' ) ] ) );

                return number_format( (double) ( $product_price / ( 1 + $vat_rate->vat_rate ) ), 4, '.', '' );
            }

            return number_format( (double) $product_price, 4, '.', '' );
        }

        /**
         * @param WC_Product $product
         *
         * @return int
         */
        private function get_product_availability( WC_Product $product ) {
            $stock = $product->get_stock_quantity();

            if ( $product->get_stock_status() === 'outofstock' ) {
                return 5;
            }

            switch ( true ) {
                case $stock <= 0:
                    return 5;
                    break;
                case $stock <= 3:
                    return 2;
                    break;
                default:
                    return 3;
                    break;
            }
        }

        /**
         * @param WC_Product $product
         *
         * @return array
         */
        private function get_product_characteristics( WC_Product $product ) {
            $query = "SELECT attribute_name,wec.emag_characteristic_id
          FROM {$this->wpdb->prefix}emkp_match_characteristics_attributes wemca
           INNER JOIN {$this->wpdb->prefix}emkp_categories_match ecm ON match_id = ecm.id
           INNER JOIN {$this->wpdb->prefix}woocommerce_attribute_taxonomies ON woocommerce_attribute_id = attribute_id
           INNER JOIN {$this->wpdb->prefix}emkp_characteristics wec ON wec.id = wemca.emag_characteristic_id
          WHERE emag_category_id = %d;";

            $mapped_characteristics = $this->wpdb->get_results( $this->wpdb->prepare( $query, [ $this->emag_category_id ] ) );

            $characteristics = [];
            foreach ( $mapped_characteristics as $mapped_characteristic ) {
                $value = $product->get_attribute( $mapped_characteristic->attribute_name );

                if ( ! $value && $product instanceof WC_Product_Variation ) {
                    $value = $this->product->get_attribute( $mapped_characteristic->attribute_name );
                }

                if ( $value ) {
                    $characteristics[] = [
                        'id'    => $mapped_characteristic->emag_characteristic_id,
                        'value' => $value,
                    ];
                }
            }

            return $characteristics;
        }

        /**
         * @param WC_Product_Variation $variation
         *
         * @return string
         */
        private function get_name_of_variation( WC_Product_Variation $variation ) {
            $variation_attributes = $variation->get_variation_attributes();

            if ( count( $variation_attributes ) < 2 ) {
                return $variation->get_name();
            }

            $bits = [ $variation->get_name() ];

            $count = 0;
            foreach ( $variation_attributes as $attribute => $value ) {
                if ( $count > 2 ) {
                    break;
                }

                $attribute_code = $this->str_replace_first( 'attribute_', '', $attribute );
                $attribute      = get_taxonomy( $attribute_code );

                if ( ! $attribute ) {
                    continue;
                }

                $value = $variation->get_attribute( $attribute_code );

                if ( ! empty( $value ) ) {
                    $bits[] = $value;
                }
                $count ++;
            }

            return implode( ', ', $bits );
        }

        /**
         * @param string $search
         * @param string $replace
         * @param string $subject
         *
         * @return string
         */
        private function str_replace_first( $search, $replace, $subject ) {
            $pos = strpos( $subject, $search );
            if ( $pos !== false ) {
                return substr_replace( $subject, $replace, $pos, strlen( $search ) );
            }

            return $subject;
        }
    }
endif;

return [
    'command_name' => 'process-products',
    'callback'     => function () {
        global $wpdb;
        $queue_limit = (int) get_option( 'emkp_product_queue_limit' );

        if ( ! $queue_limit ) {
            die( 'No queue limit! Please set it in plugin config.' );
        }

        if ( ! get_option( 'emkp_api_url', false ) || ! get_option( 'emkp_username', false ) || ! get_option( 'emkp_password', false ) ) {
            exit;
        }

        $products      = $wpdb->get_results( $wpdb->prepare( "SELECT product_id, state FROM {$wpdb->prefix}emkp_products_queue WHERE status = %s ORDER BY created_at ASC LIMIT 0, $queue_limit",
            [ 'pending' ] ) );
        $products_sent = 0;

        foreach ( $products as $product ) {
            $status  = 'failed';
            $message = null;

            $wpdb->query( $wpdb->prepare( "UPDATE {$wpdb->prefix}emkp_products_queue SET status=%s WHERE product_id = %d AND status=%s", [
                'running',
                $product->product_id,
                'pending'
            ] ) );

            try {
                $response = ( new Emkp_Product_Sender( $product->product_id, $product->state, $products_sent ) )->send();
                $status   = 'sent';
                $message  = json_encode( $response );
            } catch ( Exception $e ) {
                $message = $e->getMessage();
            }

            $wpdb->query( $wpdb->prepare( "UPDATE {$wpdb->prefix}emkp_products_queue SET `status`=%s, `exception_message`=%s WHERE `product_id` = %d AND status=%s", [
                $status,
                $message,
                $product->product_id,
                'running'
            ] ) );

            // stop if we reached the limit
            // necessary because one product might have more than one variation and a variation = an emag product
            if ( $products_sent >= $queue_limit ) {
                break;
            }
        }
    }
];
