<?php

namespace Zitec\MkpApiWrapper;

use Zitec\MkpApiWrapper\AlertManager\AlertManagerInterface;
use Zitec\MkpApiWrapper\AlertManager\NullManager;
use Zitec\MkpApiWrapper\Exceptions\FailedRequestException;
use Zitec\MkpApiWrapper\Logger\FileLogger;
use Zitec\MkpApiWrapper\Logger\LoggerInterface;
use Zitec\MkpApiWrapper\Requests\AbstractRequest;
use Zitec\MkpApiWrapper\Requests\Awb\ReadRequest;

class InstantiableClient
{
    /**
     * Marketplace API URL
     *
     * @var string
     */
    private $mkp_api_url;

    /**
     * Marketplace Username
     *
     * @var string
     */
    private $mkp_username;

    /**
     * Marketplace Password
     *
     * @var string
     */
    private $mkp_password;

    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * @var AlertManagerInterface
     */
    private $alertManager;

    /**
     * @var array
     */
    private $debugInfo = [
        'site'              => null,
        'platform'          => null,
        'version'           => 0,
        'extension_version' => 0,
    ];

    /**
     * Client constructor.
     *
     * @param string                     $mkp_api_url
     * @param string                     $mkp_username
     * @param string                     $mkp_password
     * @param LoggerInterface|null       $logger
     * @param AlertManagerInterface|null $alertManager
     */
    public function __construct(
        $mkp_api_url,
        $mkp_username,
        $mkp_password,
        LoggerInterface $logger = null,
        AlertManagerInterface $alertManager = null
    ) {
        $this->mkp_api_url = $mkp_api_url;
        $this->mkp_username = $mkp_username;
        $this->mkp_password = $mkp_password;

        $this->logger = $logger ?: new FileLogger();
        $this->alertManager = $alertManager ?: new NullManager();
    }

    /**
     * @param AbstractRequest $request
     *
     * @return array
     */
    public function sendRequest(AbstractRequest $request)
    {
        $request->validateData();

        return $this->_sendRequest($this->mkp_api_url, $this->mkp_username, $this->mkp_password, $request);
    }

    /**
     * @param string $site
     * @param string $platform
     * @param int    $version
     * @param int    $extensionVersion
     * @return $this
     */
    public function setDebugInfo($site, $platform, $version, $extensionVersion)
    {
        $this->debugInfo = [
            'site'              => $site,
            'platform'          => $platform,
            'version'           => $version,
            'extension_version' => $extensionVersion
        ];
        return $this;
    }

    /**
     * @return array
     */
    public function getDebugInfo()
    {
        return $this->debugInfo;
    }

    /**
     * @param string          $baseUrl
     * @param string          $username
     * @param string          $password
     * @param AbstractRequest $request
     *
     * @return mixed
     * @throws FailedRequestException
     */
    protected function _sendRequest($baseUrl, $username, $password, AbstractRequest $request)
    {
        $endpoint = $request->getEndpoint();
        $data = $request->getRequestData(false);
        $data['debug_info'] = $this->getDebugInfo();

        $hash = base64_encode($username . ':' . $password);

        if ($request instanceof ReadRequest) {
            $baseUrl = str_replace('/api-3', null, $baseUrl);
        }

        $url = $baseUrl . '/' . $endpoint;

        $time_start = microtime(true);

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_HEADER, false);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_TIMEOUT, 400);
        curl_setopt($ch, CURLOPT_HTTPHEADER, ['Authorization: Basic ' . $hash]);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($data));

        $response = curl_exec($ch);

        $executionTime = microtime(true) - $time_start;

        if (!$request->skipLog()) {
            $this->logger->log($url, $data, $response, $executionTime, curl_error($ch), curl_errno($ch));
        }

        if ($response === false) {
            $this->alertManager->alert('curl_error', curl_error($ch), $request);

            throw new FailedRequestException(curl_error($ch), curl_errno($ch));
        }

        curl_close($ch);

        if ($request->skipJsonDecode()) {
            return $response;
        }

        $response = json_decode($response);

        if (!is_object($response) || !property_exists($response, 'isError') || $response->isError) {
            if (property_exists($response, 'messages') && is_array($response->messages)) {
                $message = implode(', ', $response->messages);
            } else {
                $message = $response;
            }

            $this->alertManager->alert('emag_error', $message, $request);
        }

        return $response;
    }
}
