<?php

namespace Zitec\MkpApiWrapper\Requests;

use Zitec\MkpApiWrapper\Exceptions\MissingEndpointException;
use Zitec\MkpApiWrapper\Exceptions\RequestDataValidationException;
use Zitec\MkpApiWrapper\Validator;

/**
 * This is the base class for requests to Emag Marketplace.
 * All other request classes MUST extend this one.
 *
 * Class AbstractRequest
 * @package Zitec\MkpApiWrapper\Requests
 */
abstract class AbstractRequest
{
    /**
     * Resource path
     *
     * @var string
     */
    protected $_endpoint;

    /**
     * Request data
     *
     * @var array
     */
    protected $_requestData = [];

    /**
     * @var array
     */
    protected $_fieldConstraints = [];

    /**
     * @var bool
     */
    protected $ignoreLocalValidations;

    /**
     * Zitec_MkpApiWrapper_Requests_AbstractRequest constructor.
     *
     * @param array $requestData
     * @param bool  $ignoreLocalValidations
     */
    public function __construct(array $requestData = [], $ignoreLocalValidations = false)
    {
        $this->setRequestData($requestData);
        $this->ignoreLocalValidations = $ignoreLocalValidations;
    }

    /**
     * @throws MissingEndpointException
     * @return string
     */
    public function getEndpoint()
    {
        if (!$this->_endpoint) {
            throw new MissingEndpointException('Endpoint not defined for request ' . static::class);
        }

        return $this->_endpoint;
    }

    /**
     * @param array $requestData
     *
     * @return $this
     */
    public function setRequestData(array $requestData)
    {
        $this->_requestData = [
            'data' => $requestData
        ];

        return $this;
    }

    /**
     * @param bool $excludeDataKey
     * @return array
     */
    public function getRequestData($excludeDataKey = true)
    {
        if (!$excludeDataKey) {
            return $this->_requestData;
        }

        return $this->_requestData['data'];
    }

    /**
     * @return array
     */
    public function getFieldConstraints()
    {
        return $this->_fieldConstraints;
    }

    /**
     * Make sure that all requested fields exist in the request data.
     *
     * @throws RequestDataValidationException
     */
    public function validateData()
    {
        if (!$this->ignoreLocalValidations) {
            $this->_validate($this->getRequestData(false));
        }
    }

    /**
     * @return bool
     */
    public function skipJsonDecode()
    {
        return false;
    }

    /**
     * @return bool
     */
    public function skipLog()
    {
        return false;
    }

    /**
     * @param $data
     *
     * @throws RequestDataValidationException
     */
    protected function _validate($data)
    {
        $validator = new Validator($data);
        $validator->mapFieldsRules($this->getFieldConstraints());

        if (!$validator->validate()) {
            throw new RequestDataValidationException('The following errors occurred on validation for request '
                . static::class . ': ' . print_r($validator->errors(), true));
        }

    }
}
