<?php

namespace Zitec\MkpApiWrapper\Requests\ProductOffer;

use Zitec\MkpApiWrapper\Requests\AbstractRequest;

/**
 * Request definition for creating new product offer on Emag Marketplace.
 *
 * Class CreateRequest
 * @package Zitec\MkpApiWrapper\Requests\ProductOffer
 */
class CreateRequest extends AbstractRequest
{
    /**
     * API endpoint
     *
     * @var string
     */
    protected $_endpoint = 'product_offer/save';

    /**
     * Field constraints
     *
     * @var array
     */
    protected $_fieldConstraints = [
        'id'                    => ['required', 'integer', ['min', 1], ['max', 16777215]],
        'category_id'           => ['required', 'integer', ['min', 1], ['max', 65535]],
        'vendor_category_id'    => ['optional', 'integer'],
        'part_number_key'       => ['optional'],
        'name'                  => ['required', ['lengthBetween', 1, 255]],
        'part_number'           => ['required', ['lengthBetween', 1, 128]],
        'description'           => ['required', ['lengthBetween', 1, 16777215]],
        'brand'                 => ['required', ['lengthBetween', 1, 128]],
        'weight'                => ['optional', 'numeric', ['min', 0], ['max', 999999]],
        'force_images_download' => ['optional', 'boolean'],

        'images'                => ['optional', 'array'],
        'images.*.display_type' => ['optional', 'integer', ['min', 0], ['max', 2]],
        'images.*.url'          => [
            'requiredWithOptionalParent',
            'url',
            ['lengthBetween', 1, 1024],
            ['regex', '/\.(jpg|jpeg|gif|png)(?![\w.\-_])/i']
        ],

        'characteristics'         => ['optional', 'array'],
        'characteristics.*.id'    => ['requiredWithOptionalParent', 'integer', ['min', 1], ['max', 65535]],
        'characteristics.*.value' => ['requiredWithOptionalParent', ['lengthBetween', 1, 255]],

        'family'                => ['optional', 'array'],
        'family.id'             => ['requiredWithOptionalParent', 'integer'],
        'family.name'           => [['requiredIfOtherFieldNotZero', 'family.id']],
        'family.family_type_id' => [['requiredIfOtherFieldNotZero', 'family.id'], 'integer'],

        'url'      => ['required', ['lengthBetween', 1, 2014]],
        'warranty' => ['optional', 'integer', ['min', 0], ['max', 255]],

        'barcode'   => ['optional', 'array'],
        'barcode.*' => [['lengthBetween', 1, 20]],

        'attachments'       => ['optional', 'array'],
        'attachments.*.id'  => ['optional', 'integer', ['min', 1], ['max', 4294967295]],
        'attachments.*.url' => ['requiredWithOptionalParent', 'url', ['lengthBetween', 1, 1024]],

        'status'            => ['required', 'integer', ['min', 0], ['max', 1]],
        'sale_price'        => ['required', 'numeric', ['min', 1]],
        'recommended_price' => ['optional', ['min', 1],],
        'min_sale_price'    => ['optional', 'numeric', ['min', 1]],
        'max_sale_price'    => ['optional', 'numeric', ['min', 1], ['greaterThanField', 'min_sale_price']],

        'availability'                => ['required', 'array'],
        'availability.*.warehouse_id' => ['required', 'integer'],
        'availability.*.id'           => ['required', 'integer'],

        'stock'                => ['required', 'array'],
        'stock.*.warehouse_id' => ['required', 'integer'],
        'stock.*.value'        => ['required', 'integer', ['min', 0], ['max', 65535]],

        'handling_time'              => ['optional', 'array'],
        'handling_time.warehouse_id' => ['requiredWithOptionalParent', 'integer'],
        'handling_time.value'        => ['requiredWithOptionalParent', 'integer', ['min', 0], ['max', 255]],

        'start_date' => ['optional', ['dateFormat', 'Y-m-d'], ['dateAfter', '+1day'], ['dateBefore', '+60day']],
        'vat_id'     => ['required', 'integer'],
    ];

    /**
     * Override validator to accept an array of product, not just one product
     */
    public function validateData()
    {
        if ($this->ignoreLocalValidations) {
            return true;
        }

        // call parent validator if there's just one product in the array
        if (array_key_exists('id', $this->getRequestData())) {
            parent::validateData();
        } else { // validate each product from the array
            foreach ($this->getRequestData() as $product) {
                $this->_validate($product);
            }
        }
    }
}
