<?php

namespace Zitec\MkpApiWrapper;

use Zitec\MkpApiWrapper\Validator\CustomRules;

/**
 * Class Validator
 * @package Zitec\MkpApiWrapper
 */
class Validator extends \Valitron\Validator
{
    public function __construct(array $data = array(), array $fields = array(), $lang = null, $langDir = null)
    {
        parent::__construct($data, $fields, $lang, $langDir);

        $customRules = new CustomRules();
        $customRules->init();
    }

    /**
     * Add an error to error messages array
     *
     * @param string $field
     * @param string $msg
     * @param array  $params
     */
    public function error($field, $msg, array $params = array())
    {
        $newError = !array_key_exists($field, $this->_errors);
        $msg = $this->checkAndSetLabel($newError ? $field : ' and it', $msg, $params, $newError);

        $values = array();
        // Printed values need to be in string format
        foreach ($params as $param) {
            if (is_array($param)) {
                $param = "['" . implode("', '", $param) . "']";
            }
            if ($param instanceof \DateTime) {
                $param = $param->format('Y-m-d');
            } else {
                if (is_object($param)) {
                    $param = get_class($param);
                }
            }
            // Use custom label instead of field name if set
            if (is_string($params[0])) {
                if (isset($this->_labels[$param])) {
                    $param = $this->_labels[$param];
                }
            }
            $values[] = $param;
        }

        $message = vsprintf($msg, $values);
        $this->_errors[$field] = $newError ? $message : $this->_errors[$field] . $message;
    }

    /**
     * @param  string  $field
     * @param  string  $msg
     * @param  array   $params
     * @param  boolean $ucwords
     *
     * @return array
     */
    protected function checkAndSetLabel($field, $msg, $params, $ucwords = true)
    {
        if (isset($this->_labels[$field])) {
            $msg = str_replace('{field}', $this->_labels[$field], $msg);

            if (is_array($params)) {
                $i = 1;
                foreach ($params as $k => $v) {
                    $tag = '{field' . $i . '}';
                    $label = isset($params[$k]) && (is_numeric($params[$k]) || is_string($params[$k])) && isset($this->_labels[$params[$k]]) ? $this->_labels[$params[$k]] : $tag;
                    $msg = str_replace($tag, $label, $msg);
                    $i++;
                }
            }
        } else {
            $field = str_replace('_', ' ', $field);
            $words = $ucwords ? ucwords($field) : $field;
            $msg = str_replace('{field}', $words, $msg);
        }

        return $msg;
    }

    /**
     * Run validations and return boolean result
     *
     * @return boolean
     */
    public function validate()
    {
        foreach ($this->_validations as $v) {
            foreach ($v['fields'] as $field) {
                list($values, $multiple) = $this->getPart($this->_fields, explode('.', $field));

                // Don't validate if the field is not required and the value is empty
                if ($this->hasRule('optional', $field) && isset($values)) {
                    //Continue with execution below if statement
                } elseif ($v['rule'] !== 'required' && !$this->hasRule('required',
                        $field) && (!isset($values) || $values === '' || ($multiple && count($values) == 0))
                ) {
                    continue;
                }

                // Callback is user-specified or assumed method on class
                $errors = $this->getRules();
                if (isset($errors[$v['rule']])) {
                    $callback = $errors[$v['rule']];
                } else {
                    $callback = array($this, 'validate' . ucfirst($v['rule']));
                }

                if (!$multiple) {
                    $values = array($values);
                }

                $result = true;
                foreach ($values as $k => $value) {
                    $result = $result && call_user_func($callback, $field, $value, $v['params'], $this->_fields, $k);
                }

                if (!$result) {
                    $this->error($field, $v['message'], $v['params']);
                }
            }
        }

        return count($this->errors()) === 0;
    }
}
