<?php

namespace Zitec\MkpApiWrapper\Validator;

use Zitec\MkpApiWrapper\Validator;

/**
 * Class CustomRules
 * @package Zitec\MkpApiWrapper\Validator
 */
class CustomRules
{
    /**
     * Validation messages for each custom rule.
     * The key of the array represents the name of the rule and the static method that will be called for validation.
     *
     * @var array
     */
    private $_validationMessages = [
        'requiredWithOptionalParent'  => 'is required',
        'requiredIfOtherFieldNotZero' => 'is required if %s is not zero',
        'greaterThanField'            => 'has to be greater than %s',
    ];

    /**
     * Initialize custom rules that have messages defined.
     */
    public function init()
    {
        foreach ($this->_validationMessages as $rule => $message) {
            Validator::addRule($rule, [self::class, $rule], $message);
        }
    }

    /**
     * Rule use to validate the existence of an element of an array.
     *
     * Usage example:
     *      $constraints['parent_array'] = ['optional', 'array'];
     *      $constraints['parent_array.*.child_element'] = ['requiredWithOptionalParent'];
     *
     * @param string $field
     * @param string $value
     * @param array  $params
     * @param array  $fields
     *
     * @return bool
     *
     * @SuppressWarnings(PHPMD.UnusedLocalVariable)
     */
    public static function requiredWithOptionalParent($field, $value, array $params, array $fields)
    {
        $item = explode('.', $field);
        $item = array_shift($item);

        if (array_key_exists($item, $fields)) {
            if (is_null($value)) {
                return false;
            } elseif (is_string($value) && trim($value) === '') {
                return false;
            }
        }

        return true;
    }

    /**
     * Rule used to validate the existence of another field in case that the field is not zero.
     *
     * Usage example:
     *      $constraints['field'] = [['requiredIfOtherFieldNotZero', 'another_field']];
     *
     * @param string  $field
     * @param string  $value
     * @param array   $params
     * @param array   $fields
     * @param integer $k
     *
     * @return bool
     *
     * @SuppressWarnings(PHPMD.UnusedLocalVariable)
     */
    public static function requiredIfOtherFieldNotZero($field, $value, array $params, array $fields, $k)
    {
        $item = explode('.', $params[0]);

        $lastItem = $fields;
        foreach ($item as $value) {
            if ($value == '*') {
                $value = $k;
            }

            if (!array_key_exists($value, $lastItem)) {
                return false;
            }

            $lastItem = $lastItem[$value];
        }

        if (is_null($lastItem) || $lastItem === 0) {
            return false;
        }

        return true;
    }

    /**
     * Rule used to check if the numeric value of a field is greater than another one's.
     *
     * Usage example:
     *      $constraints['field'] = [['greaterThanField', 'another_field']];
     *
     * @param string $field
     * @param string $value
     * @param array  $params
     * @param array  $fields
     *
     * @return bool
     *
     * @SuppressWarnings(PHPMD.UnusedLocalVariable)
     */
    public static function greaterThanField($field, $value, array $params, array $fields)
    {
        if (!count($params) ||
            !array_key_exists($params[0], $fields) ||
            !is_numeric($fields[$params[0]]) ||
            !is_numeric($value)
        ) {
            return false;
        }

        return $value > $fields[$params[0]];
    }
}
